///###////////////////////////////////////////////////////////////////////////
//
// Burton Computer Corporation
// http://www.burton-computer.com
// http://www.cooldevtools.com
// $Id: testhdl.cc 216 2005-12-26 05:21:59Z brian $
//
// Copyright (C) 2005 Burton Computer Corporation
// ALL RIGHTS RESERVED
//
// This program is open source software; you can redistribute it
// and/or modify it under the terms of the Q Public License (QPL)
// version 1.0. Use of this software in whole or in part, including
// linking it (modified or unmodified) into other programs is
// subject to the terms of the QPL.
//
// This program is distributed in the hope that it will be useful,
// but WITHOUT ANY WARRANTY; without even the implied warranty of
// MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
// Q Public License for more details.
//
// You should have received a copy of the Q Public License
// along with this program; see the file LICENSE.txt.  If not, visit
// the Burton Computer Corporation or CoolDevTools web site
// QPL pages at:
//
//    http://www.burton-computer.com/qpl.html
//    http://www.cooldevtools.com/qpl.html
//

#include "File.h"
#include "HdlError.h"
#include "HdlToken.h"
#include "HdlStatement.h"
#include "HdlParser.h"
#include "HdlPrinter.h"
#include "HdlSyntaxChecker.h"

Ref<HdlStatement> parseFile(const char *filename)
{
  File f(filename);
  if (!f.isFile()) {
    throw runtime_error(string("no such file: ") + filename);
  }

  HdlParser parser;
  return parser.parseFile(f);
}

int main(int argc,
         const char **argv)
{
  if (argc != 2 && argc != 3) {
    cerr << "usage: testhdl hdl_filename syntax_filename" << endl;
    return 1;
  }

  try {
    Ref<HdlStatement> stmt(parseFile(argv[1]));
    HdlPrinter printer;
    printer.printStatement(cout, stmt);

    if (argc == 3) {
      Ref<HdlStatement> syntax_stmt(parseFile(argv[2]));
      printer.printStatement(cout, syntax_stmt);
      HdlSyntaxChecker checker(syntax_stmt);
      checker.checkSyntax(stmt);
      cout << "Syntax OK" << endl;
    }

  } catch (HdlError &ex) {
    cerr << "caught hdl error: " << ex.what() << endl;
    return 1;
  } catch (exception &ex) {
    cerr << "caught exception: " << ex.what() << endl;
    return 1;
  }
  return 0;
}
